/***************************************************************************
 *   Copyright (C) 2007  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 16 septembre 2006
   derniere modification le 18/03/2007 */


#ifndef __SCEERROR_H__
#define __SCEERROR_H__

#include <stdio.h>
#include <time.h>

#define Logger_Init(out) SCEError_Init(out)
#define Logger_Clear() SCEError_Clear()
#define Logger_LogCode(c)\
 SCEError_SetErrorCode_(__FILE__, __FUNCTION__, __LINE__, c)
#define Logger_Log_(a,b,c) SCEError_SetErrorCode_(a,b,c)
#define Logger_LogSrc() SCEError_SetSrc(__FILE__, __FUNCTION__, __LINE__)
#define Logger_GetMsg() SCEError_GetErrorMsg()
#define Logger_GetCode() SCEError_GetErrorCode()
#define Logger_GetCodeMsg(code) SCEError_GetCodeMsg(code)
#define Logger_GetFile() SCEError_GetFile()
#define Logger_GetFunc() SCEError_GetFunc()
#define Logger_GetLine() SCEError_GetLine()
#define Logger_GetTime() SCEError_GetTime()
#define Logger_HaveError() SCEError_HaveError()
#define Logger_HaveMsg() SCEError_HaveMsg()
#define Logger_Out() SCEError_OutStream()

/*
#define Logger_Cant_Init(m) "can't initialize"m
*/

#define Logger_LogFinish(msg)\
    Logger_LogSrc();\
    if(!Logger_HaveMsg())\
    {\
        if(msg)\
            Logger_LogMsg(msg);\
        else\
            Logger_LogMsg(Logger_GetCodeMsg(Logger_GetCode()));\
    }


#define Logger_Log(code)\
if(Logger_HaveError())\
{\
    fprintf(stderr, "%s (%d): have already a error logged!!\n",\
            __FILE__, __LINE__);\
}\
else\
    Logger_LogCode(code)


/*
#define Logger_Log(code)\
    #ifdef __DEBUG__\
    if(Logger_HaveError())\
    {\
        fprintf(stderr, "%s (%d): have already a error logged\n", __FILE__, __LINE__);\
    }\
    else\
    {\
    #endif\
        if(Logger_HaveMsg())\
        {\
        Logger_Log(code);\
        Logger_LogMsg(Logger_GetCodeMsg());
    #ifdef __DEBUG__\
    }\
    #endif
*/

#define SCE_NUM_ERRORS 8

enum sce_enum_error
{
    SCE_OUT_OF_MEMORY,
    SCE_INVALID_OPERATION,
    SCE_INVALID_POINTER,
    SCE_INVALID_ARG,
    SCE_INVALID_SIZE,
    SCE_INVALID_ENUM,
    SCE_FILE_NOT_FOUND,
    SCE_BAD_FORMAT
};
typedef enum sce_enum_error SCE_EError;



/* initialise le gestionnaire d'erreur */
int SCEError_Init(FILE*);

/* vide les erreurs stockees */
void SCEError_Clear(void);

/* sauvegarde un code d'erreur */
void SCEError_SetErrorCode_(const char*, const char*, const int, int);

/* sauvegarde un message d'erreur */
int Logger_LogMsg(const char*, ...);

/* affiche un message sur la sortie par defaut du logger */
void Logger_PrintMsg(const char*, ...);

/* sauvegarde la zone appellante de la fonction qui a echoue */
void SCEError_SetSrc(const char*, const char*, const int);

/* renvoie le message d'erreur */
const char* SCEError_GetErrorMsg(void);

/* renvoie le code d'erreur */
int SCEError_GetErrorCode(void);

/* renvoie le message d'erreur associe au code */
const char* SCEError_GetCodeMsg(int);

/* renvoie le fichier ou s'est produite l'erreur */
const char* SCEError_GetFile(void);

/* retourne le nom de la fonction ou s'est produite l'erreur */
const char* SCEError_GetFunc(void);

/* renvoie la ligne ou s'est produite l'erreur */
int SCEError_GetLine(void);

/* renvoie l'heure ou s'est produite l'erreur */
time_t SCEError_GetTime(void);

/* verifie si il y a une erreur */
int SCEError_HaveError(void);

/* verifie si un message d'erreur est disponible */
int SCEError_HaveMsg(void);

/* ecrit l'erreur dans le flux par defaut du gestionnaire */
int SCEError_OutStream(void);



#endif /* guard */
