/***************************************************************************
 *   Copyright (C) 2007  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 12 octobre 2006
   derniere modification le 26/06/2007 */


#include <SCE/SCEngine.h>


int SCE_Init(FILE *outlog, SCEflags flags)
{
    if(outlog)
        Logger_Init(outlog);
    else
    {
        fputs("error: invalid output stream of errors", stderr);
        return (SCE_ERROR);
    }
    
    /* initialisation du coeur */
    if(SCE_CInit(flags) < 0)
    {
        Logger_LogFinish("can't initialize core !");
        return (SCE_ERROR);
    }
    /***/
    
    /* il est important d'initialiser le gestionnaire de medias avant
       le gestionnaire de textures, car l'initialisation de ce dernier
       fait un appel au media manager */
    
    /* initialisation du gestionnaire de medias */
    if(SCE_Media_Init() < 0)
    {
        SCE_CQuit();
        Logger_LogFinish("can't initialize media manager");
        return (SCE_ERROR);
    }
    /***/
    
    /* initialisation du gestionnaire de ressources */
    if(SCE_Resource_Init() < 0)
    {
        SCE_CQuit();
        Logger_LogFinish("can't initialize resources manager");
        return (SCE_ERROR);
    }
    /***/
    
    /* initialisation du gestionnaire de textures */
    if(SCE_Texture_Init() < 0)
    {
        SCE_Quit();
        Logger_LogFinish("can't initialize texture manager");
        return (SCE_ERROR);
    }
    /***/
    
    /* initialisation du gestionnaire de shaders */
    if(SCE_Shader_Init() < 0)
    {
        SCE_Quit();
        Logger_LogFinish("can't initialize shader manager");
        return (SCE_ERROR);
    }
    /***/
    
    /* initialisation du gestionnaire de meshs */
    if(SCE_Model_Init() < 0)
    {
        SCE_Quit();
        Logger_LogFinish("can't initialize models manager");
        return (SCE_ERROR);
    }
    /***/
    
    return (SCE_OK);
}

int SCE_Quit(void)
{
    if(SCE_Shader_Quit() < 0)
    {
        Logger_LogFinish(NULL);
        return (SCE_ERROR);
    }
    
    if(SCE_Texture_Quit() < 0)
    {
        Logger_LogFinish(NULL);
        return (SCE_ERROR);
    }
    
    if(SCE_Resource_Quit() < 0)
    {
        Logger_LogFinish(NULL);
        return (SCE_ERROR);
    }
    
    if(SCE_Media_Quit() < 0)
    {
        Logger_LogFinish(NULL);
        return (SCE_ERROR);
    }
    
    if(SCE_CQuit() < 0)
    {
        Logger_LogFinish(NULL);
        return (SCE_ERROR);
    }
    
    return (SCE_OK);
}
