/***************************************************************************
 *   Copyright (C) 2007  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 31 mai 2007
   derniere modification : 16/06/2007 */


#include <SCE/SCEMinimal.h>

#include <SCE/core/SCECMaterial.h>


/* revise le 16/06/2007 */
SCE_CMaterial* SCE_CCreateMaterial(void)
{
    SCE_CMaterial *mat = NULL;
    
    mat = SCE_malloc(sizeof *mat);
    if(mat == NULL)
    {
        Logger_LogSrc();
        return (NULL);
    }
    
    /* NOTE: et Vector c'est pour les chiens ? */
    
    mat->colors[0][0] = mat->colors[0][1] =
    mat->colors[0][2] = mat->colors[0][3] = 1.0f;
    
    mat->colors[1][0] = mat->colors[1][1] =
    mat->colors[1][2] = mat->colors[1][3] =
    mat->colors[2][0] = mat->colors[2][1] =
    mat->colors[2][2] = mat->colors[2][3] =
    mat->colors[3][0] = mat->colors[3][1] =
    mat->colors[3][2] = mat->colors[3][3] = 0.0f;
    
    mat->colors[4][0] = 64.0f;
    
    return (mat);
}

void SCE_CDeleteMaterial(SCE_CMaterial *mat)
{
    SCE_free(mat);
}


void SCE_CSetColorMaterial(SCE_CMaterial *mat, SCEenum type,
                           float r, float g, float b, float a)
{
    if(mat != NULL)
    {
        mat->colors[type][0] = r;
        mat->colors[type][1] = g;
        mat->colors[type][2] = b;
        mat->colors[type][3] = a;
    }
}

void SCE_CSetColorMaterialv(SCE_CMaterial *mat, SCEenum type, float *color)
{
    SCE_CSetColorMaterial(mat, type, color[0], color[1], color[2], color[3]);
}


/* revise le 16/06/2007 */
void SCE_CUseMaterial(SCE_CMaterial *mat)
{
    static int not_defaults = SCE_FALSE;
    
    /* couleurs par defaut */
    static float diffuse[4] = {1.0f, 1.0f, 1.0f, 1.0f};
    static float specular[4] = {0.0f, 0.0f, 0.0f, 0.0f};
    static float ambient[4] = {0.0f, 0.0f, 0.0f, 0.0f};
    static float emission[4] = {0.0f, 0.0f, 0.0f, 0.0f};
    static float shininess[1] = {64.0f};
    
    if(mat != NULL)
    {
        glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,
                           mat->colors[SCE_DIFFUSE_COLOR]);
        glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR,
                           mat->colors[SCE_SPECULAR_COLOR]);
        glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT,
                           mat->colors[SCE_AMBIENT_COLOR]);
        glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION,
                           mat->colors[SCE_EMISSIVE_COLOR]);
        glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS,
                           mat->colors[SCE_SHININESS_COLOR]);
        
        not_defaults = SCE_TRUE;
    }
    else if(not_defaults)
    {
        /* on place les valeurs par defaut */
        glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
        glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
        glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
        glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emission);
        glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, shininess);
        
        not_defaults = SCE_FALSE;
    }
}
