/***************************************************************************
 *   Copyright (C) 2007  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 3 novembre 2006
   derniere modification le 27/06/2007 */


#include <SCE/SCEMinimal.h>

#include <SCE/core/SCECBuffers.h>

#include <SCE/interface/SCEMesh.h>

#include <SCE/interface/SCESMLoader.h>


/* revise le 27/06/2007 */
void* SCE_SM_Load(FILE *file, const char *filename, void *nb_objs)
{
    SCE_SMesh **m = NULL;
    int i, j;
    int *n_objs = nb_objs;  /* affectation importante ! */
    int unsused;            /* variable pointee par n_objs si nb_objs = NULL */
    
    /* identifiants du tampon */
    int buf1, buf2;
    
    /* beurk (pour la compatibilite, a virer a terme) */
    float ambient[4];
    float diffuse[4];
    float specular[4];
    float shininess;
    
    /* donnees */
    unsigned int vsize;
    unsigned int nsize;
    unsigned int tsize;
    
    GLenum type;
    
    float *vertices = NULL;
    float *texcoords = NULL;
    float *normals = NULL;
    
    unsigned int lines_count;
    unsigned int triangles_count;
    unsigned int quads_count;
    
    /* les indices sont de type void,
       plus de flexibilité au niveau du type des indices : short ou int */
    void *triangles_index = NULL;
    void *quads_index = NULL;
    /***/
    
    
    /* inutilise */
    filename = NULL;
    
    /* verification de l'argument */
    if(n_objs == NULL)
        n_objs = &unsused;
    
    /* jump magic */
    fseek(file, sizeof(int), SEEK_SET);
    
    /* recuperation du nombre de meshs */
    fread(n_objs, sizeof(unsigned int), 1, file);
    if(*n_objs <= 0)
    {
        Logger_Log(SCE_BAD_FORMAT);
        return (NULL);
    }
    /***/
    
    
    /* allocation */
    m = SCE_malloc(*n_objs);
    if(m == NULL)
    {
        Logger_LogSrc();
        return (NULL);
    }
    /***/
    
    /* macro de suppression de tout la tatoin alloue */
    #define SCE_SM_DELALL()\
    {\
        Logger_LogSrc();\
        SCE_free(quads_index);\
        SCE_free(triangles_index);\
        SCE_free(normals);\
        SCE_free(vertices);\
        for(j=i; j>=0; j--)\
            SCE_Mesh_Delete(m[j]);\
        return (NULL);\
    }
    
    for(i=0; i<*n_objs; i++)
    {
        /* creation du mesh */
        m[i] = SCE_Mesh_Create();
        if(m[i] == NULL)
            SCE_SM_DELALL()
        
        /* remise a NULL des pointeurs */
        vertices = normals = texcoords = triangles_index = quads_index = NULL;
        
        /* donnes vectorielles */
        fread(&vsize, sizeof(unsigned int), 1, file);
        fread(&nsize, sizeof(unsigned int), 1, file);
        fread(&tsize, sizeof(unsigned int), 1, file);
        
        vertices = SCE_malloc(vsize * sizeof *vertices);
        if(vertices == NULL && vsize > 0)
            SCE_SM_DELALL()
        normals = SCE_malloc(nsize * sizeof *normals);
        if(normals == NULL && nsize > 0)
            SCE_SM_DELALL()
        texcoords = SCE_malloc(tsize * sizeof *texcoords);
        if(texcoords == NULL && tsize > 0)
            SCE_SM_DELALL()
        
        fread(vertices, sizeof(float), vsize, file);
        fread(normals, sizeof(float), nsize, file);
        fread(texcoords, sizeof(float), tsize, file);
        /***/
        
        
        /* indices */
        fread(&lines_count, sizeof(unsigned int), 1, file); /* TODO a virer a terme */
        fread(&triangles_count, sizeof(unsigned int), 1, file);
        fread(&quads_count, sizeof(unsigned int), 1, file); /* NOTE... et ca aussi .. ? */
        
        if((vsize/3) < 65536)
            triangles_index = SCE_malloc(triangles_count * sizeof(unsigned short));
        else
            triangles_index = SCE_malloc(triangles_count * sizeof(unsigned int));
        
        if(triangles_index == NULL && triangles_count > 0)
            SCE_SM_DELALL()
        
        if((vsize/3) < 65536)
            quads_index = SCE_malloc(quads_count * sizeof(unsigned short));
        else
            quads_index = SCE_malloc(quads_count * sizeof(unsigned int));
        
        if(quads_index == NULL && quads_count > 0)
            SCE_SM_DELALL()
        
        if((vsize/3) < 65536)
        {
            fread((unsigned short*)triangles_index, sizeof(unsigned short), triangles_count, file);
            fread((unsigned short*)quads_index, sizeof(unsigned short), quads_count, file);
        }
        else
        {
            fread((unsigned int*)triangles_index, sizeof(unsigned int), triangles_count, file);
            fread((unsigned int*)quads_index, sizeof(unsigned int), quads_count, file);
        }
        /***/
        
        /* materiau */ /* TODO a virer a terme */
        fread(ambient, sizeof(float), 4, file);
        fread(diffuse, sizeof(float), 4, file);
        fread(specular, sizeof(float), 4, file);
        fread(&shininess, sizeof(float), 1, file);
        /***/
        
        if((vsize/3) < 65536)
            type = SCE_UNSIGNED_SHORT;
        else
            type = SCE_UNSIGNED_INT;
        
        
        /* construction du mesh */
        /* creation d'un tampon */
        buf1 = SCE_Mesh_AddVB(m[i], GL_STATIC_DRAW);
        /* on y place nos donnees */
        if(SCE_Mesh_AddVertices(m[i], buf1, SCE_POSITION, SCE_FLOAT,
                                3, vsize, vertices, SCE_TRUE) < 0)
            SCE_SM_DELALL()
        
        if(SCE_Mesh_AddVertices(m[i], buf1, SCE_NORMAL, SCE_FLOAT,
                                3, nsize, normals, SCE_TRUE) < 0)
            SCE_SM_DELALL()
        if(SCE_Mesh_AddVertices(m[i], buf1, SCE_TEXCOORD0, SCE_FLOAT,
                                2, tsize, texcoords, SCE_TRUE) < 0)
            SCE_SM_DELALL()
        
        /* creation d'un tampon d'indices */
        buf2 = SCE_Mesh_AddIB(m[i], GL_STATIC_DRAW, type);
        /* ajout des indices */
        if(triangles_count > 0)
        {
            if(SCE_Mesh_AddIndices(m[i], buf2, triangles_count,
                                triangles_index, SCE_TRUE) < 0)
                SCE_SM_DELALL()
            SCE_Mesh_SetRenderMode(m[i], SCE_TRIANGLES);
        }
        else
        {
            if(SCE_Mesh_AddIndices(m[i], buf2, quads_count,
                                quads_index, SCE_TRUE) < 0)
                SCE_SM_DELALL()
            SCE_Mesh_SetRenderMode(m[i], SCE_QUADS);
        }
        
        /* construction... */
        if(SCE_Mesh_Build(m[i]) < 0)
            SCE_SM_DELALL()
        
        /* activation des tampons */
        SCE_Mesh_ActiveVB(m[i], buf1, SCE_TRUE);
        SCE_Mesh_ActiveIB(m[i], buf2, SCE_TRUE);
    }
    /***/
    
    #undef SCE_SM_DELALL
    
    /* fini :) */
    return (m);
}
