/***************************************************************************
 *   Copyright (C) 2007  Antony Martin <martin.antony@yahoo.fr>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU General Public License           *
 *   as published by the Free Software Foundation; either version 2        *
 *   of the License, or (at your option) any later version.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/* Cree le : 11 mars 2007
   derniere modification le 04/07/2007 */


#include <SCE/SCEMinimal.h>

#include <SCE/utils/SCEMath.h>
#include <SCE/utils/SCEString.h>

#include <SCE/core/SCECSupport.h>
#include <SCE/core/SCECTexture.h>
#include <SCE/core/SCECFramebuffer.h>

#include <SCE/interface/SCEDDSLoader.h>
#include <SCE/interface/SCEMedia.h>
#include <SCE/interface/SCEResources.h>

#include <SCE/interface/SCETexture.h>


static unsigned int sce_tex_reduce; /* booleen: reduction des images ? */
static SCEenum sce_tex_filter;      /* filtre de redimensionnement */

/* ajoute le 14/06/2007 */
static int sce_tex_enabled;         /* booleen, true : textures activees */
/* ajoute le 10/07/2007 */
static SCE_STexture *binded = NULL; /* texture active */
static int binded_unit = 0;         /* unite de texture de la texture active */


int SCE_Texture_MediaTypeID(void)
{
    static int id = -1;
    
    if(id == -1)
        id = SCE_Media_GenTypeID();
    
    return (id);
}


/* revise le 14/06/2007 */
int SCE_Texture_Init(void)
{
    /* initialisation de devil */
    ilInit();
    iluInit();
    
    ilOriginFunc(IL_ORIGIN_LOWER_LEFT);
    ilEnable(IL_ORIGIN_SET);
    
    /* envoie des loaders au media manager */
    SCE_Media_RegisterLoader(SCE_Texture_MediaTypeID(), 0, ".png .gif .jpg "
                             ".jpeg .bmp .ico .mng .pcx .rgb .rgba .tga .tif",
                             SCE_Texture_DevILLoader);
    
    SCE_Media_RegisterLoader(SCE_Texture_MediaTypeID(), 0, ".dds",
                             SCE_Texture_DDSLoader);
    
    /* par defaut, on reduit les images lors du redimensionnement */
    sce_tex_reduce = SCE_TRUE;
    sce_tex_filter = SCE_FILTER_NEAREST;
    sce_tex_enabled = SCE_TRUE;
    
    return (SCE_OK);
}

int SCE_Texture_Quit(void)
{
    /* fermeture de devil */
    ilShutDown();
    
    return (SCE_OK);
}



/* cree une nouvelle structure de donnees d'image */
static SCE_STexData* SCE_Texture_CreateData(void)
{
    SCE_STexData *data = NULL;
    
    data = SCE_malloc(sizeof *data);
    if(data == NULL)
    {
        Logger_LogSrc();
        return (NULL);
    }
    
    data->cdata = SCE_malloc(sizeof *data->cdata);
    if(data->cdata == NULL)
    {
        SCE_free(data);
        Logger_LogSrc();
        return (NULL);
    }
    
    data->id = 0;
    data->n_data = 1;
    data->freetexels = 0;
    
    return (data);
}

/* detruit une donnee image */
static void SCE_Texture_DeleteData(SCE_STexData *data)
{
    if(data != NULL)
    {
        if(ilIsImage(data->id))
            ilDeleteImages(1, &data->id);
        
        if(data->n_data > 0 && data->cdata != NULL)
        {
            /* les suivants sont forcement des niveaux de mipmap et par la
               meme occasion, viennent du meme pointeur, on libere donc
               uniquement le premier */
            if(data->freetexels)
                SCE_free(data->cdata[0].texels);
            
            SCE_free(data->cdata);
        }
        
        SCE_free(data);
    }
}


/* revise le 10/07/2007 */
SCE_STexture* SCE_Texture_Create(SCEenum type, int w, int h)
{
    int i = 0;
    SCE_STexData *data[6] = {NULL, NULL, NULL, NULL, NULL, NULL};
    SCE_STexture *tex = NULL;
    
    tex = SCE_malloc(sizeof *tex);
    if(tex == NULL)
    {
        Logger_LogSrc();
        return (NULL);
    }
    
    for(i=0; i<6; i++)
        tex->fb[i] = NULL;
    
    tex->n_data = 0;
    tex->data = NULL;
    
    tex->loaded = tex->builded = 0;
    
    
    switch(type)
    {
        case SCE_RENDER_DEPTH:
            type = SCE_DEPTH_BUFFER;
        case SCE_RENDER_COLOR:
            /* ben oui, sinon on ignore le depth */
            if(type == SCE_RENDER_COLOR)
                type = SCE_COLOR_BUFFER;
            
            tex->fb[0] = SCE_CCreateFramebuffer();
            if(tex->fb[0] == NULL)
            {
                SCE_free(tex);
                Logger_LogSrc();
                return (NULL);
            }
            
            /* s'il s'agit d'une color, on ajoute un depth buffer */
            if(type == SCE_COLOR_BUFFER)
                SCE_CAddRenderBuffer(tex->fb[0], SCE_DEPTH_BUFFER, 0, w, h);
            
            if(SCE_CCreateRenderTexture(tex->fb[0], type, 0, 0, 0, w, h) < 0)
            {
                SCE_Texture_Delete(tex);
                Logger_LogSrc();
                return (NULL);
            }
            
            tex->tex = SCE_CGetRenderTexture(tex->fb[0], type);
            
            break;
        
        case SCE_RENDER_DEPTH_CUBE:
            type = SCE_DEPTH_BUFFER;
        case SCE_RENDER_COLOR_CUBE:
            /* ben oui, sinon on ignore le depth */
            if(type == SCE_RENDER_COLOR_CUBE)
                type = SCE_COLOR_BUFFER;
            
            tex->tex = SCE_CCreateTexture(SCE_TEX_CUBE);
            if(tex->tex == NULL)
            {
                SCE_Texture_Delete(tex);
                Logger_LogSrc();
                return (NULL);
            }
            
            for(i=0; i<6; i++)
            {
                tex->fb[i] = SCE_CCreateFramebuffer();
                if(tex->fb[i] == NULL)
                {
                    SCE_Texture_Delete(tex);
                    Logger_LogSrc();
                    return (NULL);
                }
                
                data[i] = SCE_Texture_CreateData();
                if(data[i] == NULL)
                {
                    /* pas besoin de supprimer les autres donnees, elles sont
                       incluses a la texture donc Delete() s'en charge */
                    SCE_Texture_Delete(tex);
                    Logger_LogSrc();
                    return (NULL);
                }
                
                data[i]->cdata[0].texels = NULL;
                data[i]->cdata[0].level = 0;
                /* le target est defini dans AddData */
                data[i]->cdata[0].w = w;
                data[i]->cdata[0].h = h;
                data[i]->cdata[0].d = 0;
                data[i]->cdata[0].type = SCE_UNSIGNED_BYTE;
                
                if(type == SCE_DEPTH_BUFFER)
                {
                    data[i]->cdata[0].format = GL_DEPTH_COMPONENT;
                    data[i]->cdata[0].internal = GL_DEPTH_COMPONENT24;
                }
                else
                {
                    data[i]->cdata[0].format = GL_RGBA;
                    data[i]->cdata[0].internal = GL_RGBA;
                }
                
                if(SCE_Texture_AddData(tex, SCE_TEX_POSX + i, data[i]) < 0)
                {
                    /* la il faut les supprimer les donnees, bicoz elles
                       ne sont pas encore incluses a la texture */
                    SCE_Texture_DeleteData(data[i]);
                    SCE_Texture_Delete(tex);
                    Logger_LogSrc();
                    return (NULL);
                }
                
                /* s'il s'agit d'une color, on ajoute un depth buffer */
                if(type == SCE_COLOR_BUFFER)
                    SCE_CAddRenderBuffer(tex->fb[i], SCE_DEPTH_BUFFER, 0, w, h);
                
                SCE_CAddRenderTexture(tex->fb[i], type,
                                      SCE_TEX_POSX + i, tex->tex, 0);
            }
            
            break;
        
        default:
            /* aucun type "render" specifie, c'est une texture normale */
            tex->tex = SCE_CCreateTexture(type);
            if(tex->tex == NULL)
            {
                SCE_free(tex);
                Logger_LogSrc();
                return (NULL);
            }
    }
    
    if(type == SCE_RENDER_COLOR_CUBE || type == SCE_TEX_CUBE ||
       type == SCE_RENDER_DEPTH_CUBE)
    {
        SCE_CUseTexture(0, tex->tex, 1);
        SCE_CTextureParam(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
        SCE_CTextureParam(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
        SCE_CTextureParam(GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);
        SCE_CUseTexture(0, tex->tex, 0);
        /* on remet la precedente texture active */
        SCE_Texture_Use(binded, binded_unit);
    }
    
    return (tex);
}

/* revise le 16/06/2007 */
void SCE_Texture_Delete(SCE_STexture *tex)
{
    unsigned int i;
    
    if(tex != NULL)
    {
        for(i=0; i<tex->n_data; i++)
        {
            switch(SCE_Resource_Delete(tex->data[i]))
            {
                case SCE_FALSE:
                    break;
                
                case SCE_ERROR:
                    Logger_LogSrc();
                default:
                    SCE_Texture_DeleteData(tex->data[i]);
            }
        }
        
        SCE_CDeleteTexture(tex->tex);
        SCE_free(tex);
    }
}


#if 0
SCEenum SCE_Texture_GetImageType(const char *ext)
{
    SCEenum type = 0;
    
    /* macro tres pratique */
    #define SCE_TEX_RESEARCHFOR(extension, constante)\
    if(strcmp(ext, extension) == 0)\
        type = constante;\
    else
    
    SCE_TEX_RESEARCHFOR(".bmp", IL_BMP)
    SCE_TEX_RESEARCHFOR(".cut", IL_CUT)
    SCE_TEX_RESEARCHFOR(".dcx", IL_DCX)
    SCE_TEX_RESEARCHFOR(".dds", IL_DDS)/*
    SCE_TEX_RESEARCHFOR(".doom", IL_DOOM)
    SCE_TEX_RESEARCHFOR(".doomfloat", IL_DOOM_FLOAT)*/
    SCE_TEX_RESEARCHFOR(".gif", IL_GIF)
    SCE_TEX_RESEARCHFOR(".hdr", IL_HDR)
    SCE_TEX_RESEARCHFOR(".ico", IL_ICO)
    SCE_TEX_RESEARCHFOR(".jpg", IL_JPG)
    SCE_TEX_RESEARCHFOR(".jpeg", IL_JPG)
    SCE_TEX_RESEARCHFOR(".mdl", IL_MDL)
    SCE_TEX_RESEARCHFOR(".mng", IL_MNG)
    SCE_TEX_RESEARCHFOR(".pcx", IL_PCX)
    SCE_TEX_RESEARCHFOR(".pix", IL_PIX)
    SCE_TEX_RESEARCHFOR(".png", IL_PNG)
    SCE_TEX_RESEARCHFOR(".pbm", IL_PNM)
    SCE_TEX_RESEARCHFOR(".pgm", IL_PNM)
    SCE_TEX_RESEARCHFOR(".ppm", IL_PNM)
    SCE_TEX_RESEARCHFOR(".psd", IL_PSD)
    SCE_TEX_RESEARCHFOR(".pxr", IL_PXR)
    SCE_TEX_RESEARCHFOR(".bw", IL_SGI)
    SCE_TEX_RESEARCHFOR(".rgb", IL_SGI)
    SCE_TEX_RESEARCHFOR(".rgba", IL_SGI)
    SCE_TEX_RESEARCHFOR(".sgi", IL_SGI)
    SCE_TEX_RESEARCHFOR(".tga", IL_TGA)
    SCE_TEX_RESEARCHFOR(".tif", IL_TIF)
    SCE_TEX_RESEARCHFOR(".tiff", IL_TIF)
    SCE_TEX_RESEARCHFOR(".wal", IL_WAL)
    SCE_TEX_RESEARCHFOR(".xpm", IL_XPM)
    {
        type = IL_TYPE_UNKNOWN;
    }
    
    #undef SCE_TEX_RESEARCHFOR
    
    return (type);
}
#endif


/* TODO: a changer, il faudra que ca soit dependant de chaque texture
         NOTE: encore que ca serait bien chiant pour pas gd chose ... */
void SCE_Texture_SetResizeFilter(SCEenum filter)
{
    sce_tex_filter = filter;
}

void SCE_Texture_ReduceImages(int reduce)
{
    sce_tex_reduce = reduce;
}

void SCE_Texture_Resize(SCE_STexture *tex, unsigned int id, int w, int h, int d)
{
    /* uniquement si c'est une image DevIL */
    if(!tex->data[id]->freetexels)
    {
        /* binding de l'image */
        ilBindImage(tex->data[id]->id);
        
        /* application du filtre */
        iluImageParameter(ILU_FILTER, sce_tex_filter);
        
        /* redimensionnement */
        iluScale(w, h, d);
        
        /* transfert des nouvelles informations TODO: a deplacer/Update.. ? */
        tex->data[id]->cdata[0].w = w;
        tex->data[id]->cdata[0].h = h;
        tex->data[id]->cdata[0].d = d;
        tex->data[id]->cdata[0].texels = ilGetData();
        
        /* NOTE: et les mipmaps ?
                (bof, s'ils sont geres en hardware osef :-° ) */
        
        ilBindImage(0);
        
        /* mise a jour des donnees */
        SCE_Texture_Update(tex);
    }
    
    /* sinon on peut pas redimensionner :D */
}


void* SCE_Texture_DevILLoader(FILE *fp, const char *fname, void *unused)
{
    SCE_STexData *data = NULL;  /* donnees de la texture */
    GLint temp;                 /* variable temporaire */
    
    
    /* inutilise */
    fp = NULL;
    unused = NULL;
    
    data = SCE_Texture_CreateData();
    if(data == NULL)
    {
        Logger_LogSrc();
        return (NULL);
    }
    
    ilGenImages(1, &data->id);
    
    /* binding de l'id devil */
    ilBindImage(data->id);
    
    /* chargement de l'image */
    if(!ilLoadImage((char*)fname))
    {
        Logger_Log(SCE_INVALID_OPERATION);
        Logger_LogMsg("DevIL can't load '%s'", fname);
        ilBindImage(0);
        return (NULL);
    }
    
    /* recuperation des informations de l'image */
    data->cdata->level = 0;
    data->cdata->w = ilGetInteger(IL_IMAGE_WIDTH);
    data->cdata->h = ilGetInteger(IL_IMAGE_HEIGHT);
    data->cdata->format = data->cdata->internal = ilGetInteger(IL_IMAGE_FORMAT);
    data->cdata->type = ilGetInteger(IL_IMAGE_TYPE);
    data->cdata->texels = ilGetData();
    
    /* verification des dimensions de l'image */
    if(!SCE_CHasCap(SCE_TEX_NON_POWER_OF_TWO) &&
       (!SCE_Math_PowerOfTwo(data->cdata->w) ||
        !SCE_Math_PowerOfTwo(data->cdata->h)))
    {
        /* non support des dimensions non puissance de 2, et c'est le cas...
           redimensionnement de l'image via DevIL */
        
        /* recherche des dimensions puissance de 2 les plus proches de w et h */
        temp = 1;
        while(data->cdata->w > temp)
            temp *= 2;
        
        data->cdata->w = temp;
        
        temp = 1;
        while(data->cdata->h > temp)
            temp *= 2;
        
        data->cdata->h = temp;
        
        if(sce_tex_reduce)
        {
            data->cdata->w /= 2;
            data->cdata->h /= 2;
        }
        
        /* application du filtre */
        iluImageParameter(ILU_FILTER, sce_tex_filter);
        
        /* redimensionnement */
        iluScale(data->cdata->w, data->cdata->h, 1);
        
        /* re-recuperation de l'adresse de l'image */
        data->cdata->texels = ilGetData();
        
        /* logization de l'operation */
        Logger_PrintMsg("your hardware does not support non-power of two "
                        "textures.\n%s of '%s' forced.\nnew dimentions: "
                        "%d*%d\n", sce_tex_reduce ? "shrinkage":"extention",
                        fname, data->cdata->w, data->cdata->h);
    }
    
    ilBindImage(0);
    
    return (data);
}


void* SCE_Texture_DDSLoader(FILE *fp, const char *fname, void *unused)
{
    SCE_STexData *data = NULL;      /* donnees de la texture */
    SCE_SDDSImage *dds_tex = NULL;  /* informations de la texture .dds */
    size_t offset = 0;              /* pour parcourir les niveaux de mipmap */
    size_t pxsize = 0;              /* taille d'un pixel (bytes) */
    unsigned int i;
    
    
    /* inutilise */
    unused = NULL;
    
    data = SCE_Texture_CreateData();
    if(data == NULL)
    {
        Logger_LogSrc();
        return (NULL);
    }
    
    /* chargement de l'image */
    dds_tex = SCE_DDS_LoadFromFile(fp, fname);
    if(dds_tex == NULL)
    {
        Logger_LogSrc();
        SCE_Texture_DeleteData(data);
        return (NULL);
    }
    
    data->n_data = dds_tex->n_levels;
    data->cdata = SCE_realloc(data->cdata, data->n_data * sizeof *data->cdata);
    if(data->cdata == NULL)
    {
        SCE_free(dds_tex->texels);
        SCE_free(dds_tex);
        SCE_Texture_DeleteData(data);
        Logger_LogSrc();
        return (NULL);
    }
    
    
    /* recuperation des informations de l'image */
    data->cdata[0].level = 0;
    data->cdata[0].w = dds_tex->width;
    data->cdata[0].h = dds_tex->height;
    data->cdata[0].internal = data->cdata[0].format = dds_tex->format;
    data->cdata[0].type = /* OSEF si c'est du compresse... */ 0;
    data->cdata[0].texels = dds_tex->texels;
    
    SCE_free(dds_tex);
    data->freetexels = 1; /* attention, ici c'est important */
    
    /* verification des dimensions de l'image */
    if(!SCE_CHasCap(SCE_TEX_NON_POWER_OF_TWO) &&
       (!SCE_Math_PowerOfTwo(data->cdata[0].w) ||
        !SCE_Math_PowerOfTwo(data->cdata[0].h)))
    {
        /* non support des dimensions non puissance de 2, et c'est le cas... */
        Logger_Log(SCE_INVALID_OPERATION);
        Logger_LogMsg("your hardware does not support non-power of "
                      "two textures, and I does not dupport compressed "
                      "texture resizing. you must rescale your texture.");
        
        SCE_Texture_DeleteData(data);
        
        return (NULL);
    }
    
    pxsize = data->cdata[0].pxsize =
    (data->cdata[0].internal == GL_COMPRESSED_RGBA_S3TC_DXT1_EXT) ? 8 : 16;
    
    for(i=1; i<data->n_data; i++)
    {
        /* calcul de l'offset */
        offset =
        ((data->cdata[i-1].w+3)/4) * ((data->cdata[i-1].h+3)/4) * pxsize;
        
        /* copie des donnees globales */
        memcpy(&data->cdata[i], &data->cdata[i-1], sizeof *data->cdata);
        
        /* calcul des donnees membres (propres au <i>eme niveau de mipmap) */
        data->cdata[i].level = i;
        data->cdata[i].w /= 2;
        data->cdata[i].h /= 2;
        data->cdata[i].texels += offset;
        
        /* NOTE: flip(texels, w, h, pixels_size); !! */
    }
    
    /* NOTE: petite merdouille qu'il faudra gerer: le flip de l'image.
             pour le loader DevIL, c'est cool: DevIL s'en charge, mais ici,
             il va falloir mettre la main a la pate... */
    /** NOTE: en gros, prevoir un vrai gestionnaire d'images et de formats ...
              ou alors apprendre a se servir de DevIL correctement ! */
    
    return (data);
}


/* ajoute le 04/07/2007 */
int SCE_Texture_AddData(SCE_STexture *tex, SCEenum type, SCE_STexData *data)
{
    unsigned int id;
    unsigned int i;
    
    #ifdef __SCE_DEBUG__
    if(tex == NULL)
    {
        Logger_Log(SCE_INVALID_ARG);
        return (SCE_ERROR);
    }
    #endif
    
    /**
     * TODO: prevoir de gerer : faces de la cubemap en cas de texture cubemap,
     *                          niveaux de mipmaps en cas de texture simple.
     **/
    
    id = tex->n_data;
    tex->n_data++;
    
    tex->data = SCE_realloc(tex->data, tex->n_data * sizeof *tex->data);
    if(tex->data == NULL)
    {
        Logger_LogSrc();
        tex->n_data--;
        return (SCE_ERROR);
    }
    
    tex->data[id] = data;
    
    /* pour chaque sous-image (mipmap) */
    for(i=0; i<tex->data[id]->n_data; i++)
    {
        /* on affecte le type de l'image aux donnees */
        tex->data[id]->cdata[i].target = type;
        
        /* on associe l'image a la texture */
        SCE_CAssociateTextureData(tex->tex, &tex->data[id]->cdata[i]);
    }
    
    /* on retourne l'identifiant sous lequel a ete enregistree l'image */
    return (id);
}

/* revise le 04/07/2007 */
int SCE_Texture_AddImage(SCE_STexture *tex, SCEenum type, const char *fname)
{
    int code;
    SCE_STexData *data;
    
    #ifdef __SCE_DEBUG__
    if(tex == NULL)
    {
        Logger_Log(SCE_INVALID_ARG);
        return (SCE_ERROR);
    }
    #endif
    
    data = SCE_Resource_Load(fname, NULL, NULL);
    if(data == NULL)
    {
        if(Logger_HaveError())
            Logger_LogSrc();
        else
            Logger_Log(SCE_INVALID_ARG);
        
        return (SCE_ERROR);
    }
    
    code = SCE_Texture_AddData(tex, type, data);
    
    if(code < 0)
    {
        Logger_LogSrc();
        return (SCE_ERROR);
    }
    
    return (code);
}


int SCE_Texture_Build(SCE_STexture *tex, int use_mipmap)
{
    int hw_mipmap = 0;
    unsigned int i, j;
    
    #ifdef __SCE_DEBUG__
    if(tex == NULL)
    {
        Logger_Log(SCE_INVALID_ARG);
        return (SCE_ERROR);
    }
    #endif
    
    
    /* envoi des donnees (images) au coeur */
    for(i=0; i<tex->n_data; i++)
    {
        for(j=0; j<tex->data[i]->n_data; j++)
        {
            SCE_CAddTextureData(tex->tex, &tex->data[i]->cdata[j]);
        }
    }
    
    /* par defaut, le moteur tente de faire generer les mipmaps en hardware */
    if(use_mipmap && SCE_CHasCap(SCE_TEX_HW_GEN_MIPMAP))
        hw_mipmap = 1;
    
    /* construction */
    if(SCE_CBuildTexture(tex->tex, use_mipmap, hw_mipmap) < 0)
    {
        Logger_LogSrc();
        return (SCE_ERROR);
    }
    
    /* installation d'un filtre */
    if(use_mipmap || tex->tex->spec_mipmap)
        SCE_CSetTextureFilter(tex->tex, SCE_TEX_TRILINEAR);
    else
        SCE_CSetTextureFilter(tex->tex, SCE_TEX_LINEAR);
    
    return (SCE_OK);
}


int SCE_Texture_Update(SCE_STexture *tex)
{
    #ifdef __SCE_DEBUG__
    if(tex == NULL)
    {
        Logger_Log(SCE_INVALID_ARG);
        return (SCE_ERROR);
    }
    #endif
    
    if(SCE_CUpdateTexture(tex->tex) < 0)
    {
        Logger_LogSrc();
        return (SCE_ERROR);
    }
    
    return (SCE_OK);
}


/* revise le 15/06/2007 */
SCE_STexture* SCE_Texture_CreateFromFile(const char *fname, int use_mipmap)
{
    SCE_STexture *tex = NULL;
    
    /* texture 2D par defaut... les textures 3D dans un fichier ca
       se voir rarement (idem pour les cube maps) */
    tex = SCE_Texture_Create(SCE_TEX_2D, 0, 0);
    if(tex == NULL)
    {
        Logger_LogSrc();
        return (NULL);
    }
    
    SCE_Texture_AddImage(tex, SCE_TEX_2D, fname);
    
    /* construction de la texture et mise a jour des donnees */
    if(SCE_Texture_Build(tex, use_mipmap) < 0)
    {
        SCE_Texture_Delete(tex);
        Logger_LogSrc();
        return (NULL);
    }
    if(SCE_Texture_Update(tex) < 0)
    {
        SCE_Texture_Delete(tex);
        Logger_LogSrc();
        return (NULL);
    }
    
    return (tex);
}


/* ajoute le 14/06/2007 */
void SCE_Texture_Active(int enabled)
{
    sce_tex_enabled = enabled;
}
/* ajoute le 14/06/2007 */
void SCE_Texture_Enable(void)
{
    sce_tex_enabled = SCE_TRUE;
}
/* ajoute le 14/06/2007 */
void SCE_Texture_Disable(void)
{
    sce_tex_enabled = SCE_FALSE;
}

/* revise le 10/07/2007 */
void SCE_Texture_Use(SCE_STexture *tex, unsigned int unit)
{
    if(sce_tex_enabled)
    {
        /* desactivation de la precedente texture */
        SCE_CUseTexture(unit, NULL, 0);
        
        if(tex != NULL)
        {
            /* activation */
            SCE_CUseTexture(unit, tex->tex, 1);
        }
        
        binded = tex;
        binded_unit = unit;
    }
}


/* ajoute le 04/07/2007 */
void SCE_Texture_RenderTo(SCE_STexture *tex, SCEuint cubeface)
{
    if(tex != NULL)
    {
        /* s'il s'agit d'une cubemap */
        if(tex->tex->target == SCE_TEX_CUBE)
        {
            /* on prend en compte 'cubeface' */
            SCE_CUseFramebuffer(tex->fb[cubeface], NULL);
        }
        else
            SCE_CUseFramebuffer(tex->fb[0], NULL);
    }
    else
        SCE_CUseFramebuffer(NULL, NULL);
}
